/*:
 * @plugindesc リージョンに色をつけ、接触時イベントを起動します。
 * @author liply
 *
 * @help 以下のプラグインコマンドが導入されます。
 * paintRegion リージョンID 色（１６進）
 * リージョンに色を付けます。上層マップチップは対象から外れます。
 *
 * clearRegion リージョンID
 * リージョンの色付けを解除します。
 *
 * setRegionHook リージョンID 起動するコモンイベントの番号
 * リージョンに触れた際に起動するコモンイベントを指定します。
 *
 * clearRegionHook リージョンID
 * リージョンイベントを解除します。
 *
 */
(function(){
    var TILE_SIZE = 48;

    var Spriteset_Map_createCharacters = Spriteset_Map.prototype.createCharacters;
    Spriteset_Map.prototype.createCharacters = function(){
        var graphics = new PIXI.Graphics();
        graphics.z = 3;
        this._region = graphics;
        this._tilemap.addChild(graphics);

        Spriteset_Map_createCharacters.call(this);
    };

    var Spriteset_Map_prototype_update = Spriteset_Map.prototype.update;
    Spriteset_Map.prototype.update = function(){
        Spriteset_Map_prototype_update.apply(this, arguments);

        var ox = Math.floor(this._tilemap.origin.x);
        var oy = Math.floor(this._tilemap.origin.y);

        this._region.x = -ox;
        this._region.y = -oy;


        if(this._paintRegionVersion !== $gameSystem._paintRegionVersion){
            this._paintRegionVersion = $gameSystem._paintRegionVersion;

            paintRegion(this._region, $gameSystem._paintRegionIds);
        }
    };

    function paintRegion(graphics, regionIds){
        graphics.clear();
        for(var y = 0; y < $dataMap.height; ++y){
            for(var x = 0; x < $dataMap.width; ++x){
                var color = regionIds[$gameMap.regionId(x,y)];
                if(color !== undefined){
                    graphics.beginFill(color, 0.3);
                    graphics.drawRect(x*TILE_SIZE, y*TILE_SIZE, TILE_SIZE, TILE_SIZE);
                    graphics.endFill();
                }
            }
        }
    }

    var Game_Player_update = Game_Player.prototype.update;
    Game_Player.prototype.update = function(){
        Game_Player_update.apply(this, arguments);

        if(!$gameMap.isEventRunning() && $gameSystem._regionEvent){
            var event = $gameSystem._regionEvent[$gameMap.regionId(this._x, this._y)];
            if(event){
                $gameTemp.reserveCommonEvent(event);
            }
        }
    };

    var Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args){
        switch(command.toLowerCase()){
            case 'paintregion':
                if(!$gameSystem._paintRegionIds){
                    $gameSystem._paintRegionIds = {};
                    $gameSystem._paintRegionVersion = 0;
                }

                $gameSystem._paintRegionIds[args[0]] = parseInt(args[1], 16);
                $gameSystem._paintRegionVersion++;
                break;

            case 'clearregion':
                delete $gameSystem._paintRegionIds[args[0]];
                $gameSystem._paintRegionVersion++;
                break;

            case 'setregionhook':
                if(!$gameSystem._regionEvent){
                    $gameSystem._regionEvent = {};
                }

                $gameSystem._regionEvent[args[0]] = +args[1];
                break;

            case 'clearregionhook':
                delete $gameSystem._regionEvent[args[0]];
                break;
        }

        return Game_Interpreter_pluginCommand.apply(this, arguments);
    };
})();